/* WriteMenu.java
 *
 * created: Mon Jan 11 1999
 *
 * This file is part of Artemis
 *
 * Copyright (C) 1998,1999  Genome Research Limited
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/disk222/yeastpub/Repository/powmap/diana/components/WriteMenu.java,v 1.15 1999/06/15 09:37:45 kmr Exp $
 **/

package diana.components;

import diana.*;
import diana.sequence.*;

import uk.ac.sanger.pathogens.embl.Sequence;
import uk.ac.sanger.pathogens.embl.RawStreamSequence;
import uk.ac.sanger.pathogens.embl.EmblStreamSequence;
import uk.ac.sanger.pathogens.embl.GenbankStreamSequence;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 *  A menu of commands for writing out protein and bases.
 *
 *  @author Kim Rutherford
 *  @version $Id: WriteMenu.java,v 1.15 1999/06/15 09:37:45 kmr Exp $
 **/

public class WriteMenu extends SelectionMenu {
  /**
   *  Create a new WriteMenu component.
   *  @param frame The Frame that owns this Menu.
   *  @param selection_owner The owner of the selection that the commands in
   *    the menu will operate on.
   *  @param entry_group The EntryGroup object to usewhen writing a sequence.
   **/
  public WriteMenu (final Frame frame,
                    final SelectionOwner selection_owner,
                    final EntryGroup entry_group) {
    super (frame, "Write", selection_owner);
    this.entry_group = entry_group;

    final MenuItem bases_item = new MenuItem ("Bases Of Selection");
    bases_item.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent event) {
        writeBases (false);
      }
    });

    add (bases_item);

    final MenuItem aa_item = new MenuItem ("Amino Acids Of Selected Features");
    aa_item.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent event) {
        writeAminoAcids ();
      }
    });

    add (aa_item);

    final MenuItem pir_item =
      new MenuItem ("PIR Database Of Selected Features");
    pir_item.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent event) {
        writePIRDataBase ();
      }
    });

    add (pir_item);

    final MenuItem upstream_bases_item =
      new MenuItem ("Upstream Bases Of Selected Features");
    upstream_bases_item.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent event) {
        writeUpstreamBases ();
      }
    });

    add (upstream_bases_item);

    final MenuItem downstream_bases_item =
      new MenuItem ("Downstream Bases Of Selected Features");
    downstream_bases_item.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent event) {
        writeDownstreamBases ();
      }
    });

    add (downstream_bases_item);

    addSeparator ();

    final MenuItem write_raw_item =
      new MenuItem ("Write All Bases (Raw)");
    write_raw_item.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent event) {
        writeRawSequence (false);
      }
    });

    add (write_raw_item);

    final MenuItem write_fasta_item =
      new MenuItem ("Write All Bases (Fasta)");
    write_fasta_item.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent event) {
        writeRawSequence (true);
      }
    });

    add (write_fasta_item);

    final MenuItem write_embl_item =
      new MenuItem ("Write All Bases (EMBL)");
    write_embl_item.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent event) {
        writeEmblSequence ();
      }
    });

    add (write_embl_item);

    final MenuItem write_genbank_item =
      new MenuItem ("Write All Bases (GENBANK)");
    write_genbank_item.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent event) {
        writeGenbankSequence ();
      }
    });

    add (write_genbank_item);

    addSeparator ();

    final MenuItem codon_usage_item =
      new MenuItem ("Write Codon Usage of Selected Features");
    codon_usage_item.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent event) {
        writeCodonUsage ();
      }
    });

    add (codon_usage_item);
  }

  /**
   *  Write a PIR database of the selected features to a file choosen by the
   *  user.
   **/
  private void writePIRDataBase () {
    if (! checkForSelectionFeatures ()) {
      return;
    }

    final File write_file =
      getWriteFileName ("Select a PIR output file name ...", "cosmid.pir");

    if (write_file == null) {
      return;
    }

    try {
      final FileWriter writer = new FileWriter (write_file);

      final FeatureVector features_to_write =
        getSelection ().getAllFeatures ();

      for (int i = 0 ; i < features_to_write.size () ; ++i) {
        final Feature selection_feature = features_to_write.elementAt (i);

        selection_feature.writePIROfFeature (writer);
      }

      writer.close ();
    } catch (IOException e) {
      new MessageDialog (getParentFrame (),
                         "error while writing: " + e.getMessage ());
    }
  }

  /**
   *  Write the amino acid symbols of the selected features to a file choosen
   *  by the user.
   **/
  private void writeAminoAcids () {
    if (! checkForSelectionFeatures ()) {
      return;
    }

    final File write_file =
      getWriteFileName ("Select an output file name..", "amino_acids");

    if (write_file == null) {
      return;
    }

    try {
      final FileWriter writer = new FileWriter (write_file);

      final FeatureVector features_to_write =
        getSelection ().getAllFeatures ();

      for (int i = 0 ; i < features_to_write.size () ; ++i) {
        final Feature selection_feature = features_to_write.elementAt (i);

        selection_feature.writeAminoAcidsOfFeature (writer);
      }

      writer.close ();
    } catch (IOException e) {
      new MessageDialog (getParentFrame (),
                         "error while writing: " + e.getMessage ());
    }
  }

  /**
   *  Write the bases of the selection to a file choosen by the user.
   *  @param fasta_format if true then a fasta header line will be added at
   *    the start of each sequence in the output file.  There will be one
   *    header for each thing in the selection.
   **/
  private void writeBases (final boolean fasta_format) {
    final MarkerRange marker_range = getSelection ().getMarkerRange ();

    if (marker_range == null) {
      writeFeatureBases (fasta_format);
    } else {
      final File write_file =
        getWriteFileName ("Select an output file name ...", "raw_sequence");

      if (write_file == null) {
        return;
      }

      final String selection_bases = Strand.markerRangeBases (marker_range);

      try {
        final PrintWriter writer =
          new PrintWriter (new FileWriter (write_file));

        if (fasta_format) {
          writer.println ("> " + write_file.getName () + "  length: " +
                          entry_group.getSequenceLength ());
        }

        wrapAndWrite (writer, selection_bases, 60);

        writer.close ();
      } catch (IOException e) {
        new MessageDialog (getParentFrame (),
                           "error while writing: " + e.getMessage ());
      }
    }
  }

  /**
   *  Write the bases of the selected features to a file choosen by the user
   *  or show a message and return immediately if there are no selected
   *  features.
   *  @param fasta_format if true then a fasta header line will be added at
   *    the start of each sequence in the output file.  There will be one
   *    header in the output for each feature in the selection.
   **/
  private void writeFeatureBases (final boolean fasta_format) {
    if (! checkForSelectionFeatures ()) {
      return;
    }

    final File write_file =
      getWriteFileName ("Select an output file name ...", "bases");

    if (write_file == null) {
      return;
    }

    try {
      final FileWriter writer = new FileWriter (write_file);

      final FeatureVector features_to_write =
        getSelection ().getAllFeatures ();

      for (int i = 0 ; i < features_to_write.size () ; ++i) {
        final Feature selection_feature = features_to_write.elementAt (i);

        selection_feature.writeBasesOfFeature (writer);
      }

      writer.close ();
    } catch (IOException e) {
      new MessageDialog (getParentFrame (),
                         "error while writing: " + e.getMessage ());
    }
  }

  /**
   *  Write the upstream bases of the selected features to a file choosen by
   *  the user.  The user can also choose the number of upstream base to
   *  write.
   **/
  private void writeUpstreamBases () {
    if (! checkForSelectionFeatures ()) {
      return;
    }

    final TextRequester text_requester =
      new TextRequester ("write how many bases upstream of each feature?",
                         18, "");

    text_requester.addTextRequesterListener (new TextRequesterListener () {
      public void actionPerformed (final TextRequesterEvent event) {
        final String base_count_string = event.getRequesterText ().trim ();

        if (base_count_string.length () == 0) {
          new MessageDialog (getParentFrame (), "no bases written");
          return;
        }

        try {
          final int base_count =
            Integer.valueOf (base_count_string).intValue ();


          final File write_file =
            getWriteFileName ("Select an output file name ...", "up_bases");

          if (write_file == null) {
            return;
          }

          try {
            final FileWriter writer = new FileWriter (write_file);

            final FeatureVector features_to_write =
              getSelection ().getAllFeatures ();

            for (int i = 0 ; i < features_to_write.size () ; ++i) {
              final Feature selection_feature =
                features_to_write.elementAt (i);

              selection_feature.writeUpstreamBases (writer, base_count);
            }

            writer.close ();
          } catch (IOException e) {
            new MessageDialog (getParentFrame (),
                               "error while writing: " + e.getMessage ());
          }
        } catch (NumberFormatException e) {
          new MessageDialog (getParentFrame (),
                             "this is not a number: " + base_count_string);
        }
      }
    });

    text_requester.show ();
  }

  /**
   *  Write the downstream bases of the selected features to a file choosen by
   *  the user.  The user can also choose the number of downstream base to
   *  write.
   **/
  private void writeDownstreamBases () {
    if (! checkForSelectionFeatures ()) {
      return;
    }

    final TextRequester text_requester =
      new TextRequester ("write how many bases downstream of each feature?",
                         18, "");

    text_requester.addTextRequesterListener (new TextRequesterListener () {
      public void actionPerformed (final TextRequesterEvent event) {
        final String base_count_string = event.getRequesterText ().trim ();

        if (base_count_string.length () == 0) {
          new MessageDialog (getParentFrame (), "no bases written");
          return;
        }

        try {
          final int base_count =
            Integer.valueOf (base_count_string).intValue ();

          final File write_file =
            getWriteFileName ("Select an output file name ...", "down_bases");

          if (write_file == null) {
            return;
          }

          try {
            final FileWriter writer = new FileWriter (write_file);

            final FeatureVector features_to_write =
              getSelection ().getAllFeatures ();

            for (int i = 0 ; i < features_to_write.size () ; ++i) {
              final Feature selection_feature =
                features_to_write.elementAt (i);

              selection_feature.writeDownstreamBases (writer, base_count);
            }

            writer.close ();
          } catch (IOException e) {
            new MessageDialog (getParentFrame (),
                               "error while writing: " + e.getMessage ());
          }
        } catch (NumberFormatException e) {
          new MessageDialog (getParentFrame (),
                             "this is not a number: " + base_count_string);
        }
      }
    });

    text_requester.show ();
  }


  /**
   *  Write the bases from entry_group as raw sequence.
   *  @param fasta_format if true then a fasta header line will be added at
   *    the start of the output file.
   **/
  private void writeRawSequence (final boolean fasta_format) {
    final File write_file =
      getWriteFileName ("Select an output file name ...",
                        fasta_format ? "sequence.dna" : "sequence.seq");

    if (write_file == null) {
      return;
    }

    try {
      final PrintWriter writer = new PrintWriter (new FileWriter (write_file));

      if (fasta_format) {
        writer.println ("> " + write_file.getName () + "  length: " +
                        entry_group.getSequenceLength ());
      }

      final Sequence sequence = entry_group.getBases ().getSequence ();

      (new RawStreamSequence (sequence)).writeToStream (writer, true);
      
      writer.close ();
    } catch (IOException e) {
      new MessageDialog (getParentFrame (),
                         "error while writing: " + e.getMessage ());
    }
  }

  /**
   *  As the user for a file name, then write the bases from entry_group to
   *  that file as embl sequence.
   **/
  private void writeEmblSequence () {
    final File write_file =
      getWriteFileName ("Select an output file name ...", "sequence.embl");

    if (write_file == null) {
      return;
    }

    try {
      final PrintWriter writer = new PrintWriter (new FileWriter (write_file));

      final Sequence sequence = entry_group.getBases ().getSequence ();

      (new EmblStreamSequence (sequence)).writeToStream (writer, true);
      
      writer.close ();
    } catch (IOException e) {
      new MessageDialog (getParentFrame (),
                         "error while writing: " + e.getMessage ());
    }
  }

  /**
   *  As the user for a file name, then write the bases from entry_group to
   *  that file as Genbank sequence.
   **/
  private void writeGenbankSequence () {
    final File write_file =
      getWriteFileName ("Select an output file name ...", "sequence.genbank");

    if (write_file == null) {
      return;
    }

    try {
      final PrintWriter writer = new PrintWriter (new FileWriter (write_file));

      final Sequence sequence = entry_group.getBases ().getSequence ();

      (new GenbankStreamSequence (sequence)).writeToStream (writer, true);
      
      writer.close ();
    } catch (IOException e) {
      new MessageDialog (getParentFrame (),
                         "error while writing: " + e.getMessage ());
    }
  }

  /**
   *  Write a String object to a PrintWriter object, wrapping it a the given
   *  coloumn.
   *  @param writer The String will be written to this object.
   *  @param string The String to write.
   *  @param wrap_column The lines of output will be no longer than this many
   *    characters.
   **/
  private void wrapAndWrite (final PrintWriter writer,
                             final String string,
                             final int wrap_column) {
    for (int i = 0 ; i <= string.length () / wrap_column ; ++i) {
      final int first_index = i * wrap_column;
      int last_index = (i + 1) * wrap_column;

      if (last_index > string.length ()) {
        last_index = string.length ();
      }

      writer.println (string.substring (first_index, last_index));
    }
  }

  /**
   *  Popup a requester and ask for a file name to write to.
   *  @param title The title of the new FileDialog Frame.
   *  @param default_name The name to put in the FileDialog as a default.
   **/
  private File getWriteFileName (final String title,
                                 final String default_name) {
    final FileDialog dialog =
      new FileDialog (getParentFrame (), title, FileDialog.SAVE);

    dialog.setFile (default_name);

    dialog.setVisible (true);

    if (dialog.getFile () == null) {
      return null;
    } else {
      if (dialog.getFile ().length () == 0) {
        new MessageDialog (getParentFrame (), "no file name given for write");
        return null;
      } else {
        final String file_name = dialog.getDirectory () +
          java.io.File.separator + dialog.getFile ();

        System.out.println ("trying to write to: " +  file_name);

        return new File (file_name);
      }
    }
  }

  /**
   *  Write a table of codon usage for the selected features to a file choosen
   *  by the user.
   **/
  private void writeCodonUsage () {
    if (! checkForSelectionFeatures ()) {
      return;
    }

    final File write_file =
      getWriteFileName ("Select a codon usage output file ...", "usage");

    if (write_file == null) {
      return;
    }

    try {
      final PrintWriter writer = new PrintWriter (new FileWriter (write_file));

      final int [] [] [] codon_counts = new int [4][4][4];

      final FeatureVector features_to_write =
        getSelection ().getAllFeatures ();

      int codon_total = 0;

      for (int i = 0 ; i < features_to_write.size () ; ++i) {
        final Feature selection_feature = features_to_write.elementAt (i);

        for (int base1 = 0 ; base1 < 4 ; ++base1) {
          for (int base2 = 0 ; base2 < 4 ; ++base2) {
            for (int base3 = 0 ; base3 < 4 ; ++base3) {
              codon_counts[base1][base2][base3] +=
                selection_feature.getCodonCount (base1, base2, base3);
            }
          }
        }

        codon_total += selection_feature.getTranslationBasesLength () / 3;
      }

      for (int base1 = 0 ; base1 < 4 ; ++base1) {
        for (int base3 = 0 ; base3 < 4 ; ++base3) {
          final StringBuffer buffer = new StringBuffer ();

          for (int base2 = 0 ; base2 < 4 ; ++base2) {
            buffer.append (Bases.letter_index[base1]);
            buffer.append (Bases.letter_index[base2]);
            buffer.append (Bases.letter_index[base3]);
            buffer.append (' ');

            final float per_thousand;

            if (codon_total > 0) {
              per_thousand =
                10000 * codon_counts[base1][base2][base3] / codon_total *
                1.0F / 10;
            } else {
              per_thousand = 0.0F;
            }

            buffer.append (per_thousand);
            buffer.append ("( ").append (codon_counts[base1][base2][base3]);
            buffer.append (")  ");
          }

          writer.println (buffer.toString ());
        }
      }

      writer.close ();
    } catch (IOException e) {
      new MessageDialog (getParentFrame (),
                         "error while writing: " + e.getMessage ());
    }
  }

  /**
   *  The EntryGroup object that was passed to the constructor.
   **/
  final EntryGroup entry_group;
}
